#region usings
using System;
using System.ComponentModel.Composition;
using System.Collections.Generic;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{	
	[PluginInfo(Name = "Memoize", Category = "Value", Version = "Double_Double_Bool_Double", 
				Help = "Memoize a Results of a Function", Tags = "", 
				Bugs = "To make this work with different functions we either need to empty the dictionary when a new function is connected or make the function part of the key.")]
	public class Double_Double_Bool_DoubleValueMemoizeNode : IPluginEvaluate
	{
		// we accept any function that has the signature 
		// double -> double -> bool -> double
		[Input("Func")]
		ISpread<Func<double, double, bool, double>> FFunc;
		
		[Input("X")]
		ISpread<double> FX;
		
		[Input("Y")]
		ISpread<double> FY;
		
		[Input("Ad")]
		ISpread<bool> FAd;		

		[Output("Output")]
		ISpread<double> FOutput;

		Dictionary<Tuple<double, double, bool>, double> Results = 
			new Dictionary<Tuple<double, double, bool>, double>();
		
		public void Evaluate(int SpreadMax)
		{
			FOutput.SliceCount = SpreadMax;

			for (int i = 0; i < SpreadMax; i++)
			{
				// generate a key that is made of 
				// current x, current y, current ad input value
				Tuple<double, double, bool> key = 
				  new Tuple<double, double, bool>(FX[i], FY[i], FAd[i]);
				
				// the result of the function will be stored in this variable
				double result;
				
				// lets have a look if we already have a result of the function for 
				// those three input values.
				// if we have a result we dont need to call the function
				// if have no result we need to call the function and store the result in the dictionary
				if (!Results.TryGetValue(key, out result))
				{
					result = FFunc[i](FX[i], FY[i], FAd[i]);
					Results[key] = result;
				}			
				
				FOutput[i] = result;
			}
		}
	}
}
